@@ -53,7 +53,7 @@ int x11_install_info(hd_data_t *hd_data);
 int oem_install_info(hd_data_t *hd_data);
 void dump_packages(hd_data_t *hd_data);
 
-void do_hw(hd_data_t *hd_data, FILE *f, hd_hw_item_t hw_item);
+void do_hw(hd_data_t *hd_data, FILE *f, int hw_item);
 void do_hw_multi(hd_data_t *hd_data, FILE *f, hd_hw_item_t *hw_items);
 void do_short(hd_data_t *hd_data, hd_t *hd, FILE *f);
 void do_test(hd_data_t *hd_data);
@@ -322,13 +322,15 @@ int main(int argc, char **argv)
 	  break;
 
         case 1000 ... 1100:
-          if(hw_items < (int) (sizeof hw_item / sizeof *hw_item) - 1)
+          if(hw_items < sizeof hw_item / sizeof *hw_item - 1) {
             hw_item[hw_items++] = i - 1000;
+          }
           break;
 
         case 2000 ... 2005:
-          if(hw_items < (int) (sizeof hw_item / sizeof *hw_item) - 1)
+          if(hw_items < sizeof hw_item / sizeof *hw_item - 1) {
             hw_item[hw_items++] = i;
+          }
           break;
 
         default:
@@ -492,7 +494,11 @@ int main(int argc, char **argv)
 }
 
 
-void do_hw(hd_data_t *hd_data, FILE *f, hd_hw_item_t hw_item)
+/*
+ * hw_item might be either a 'real' hd_hw_item_t or some number >= 2000 used
+ * for some special probe runs.
+ */
+void do_hw(hd_data_t *hd_data, FILE *f, int hw_item)
 {
   hd_t *hd, *hd0;
   int smp = -1, uml = 0, xen = 0, i;
@@ -511,7 +517,7 @@ void do_hw(hd_data_t *hd_data, FILE *f, hd_hw_item_t hw_item)
     case 2001:
     case 2003:
       i = -1;
-      switch((int) hw_item) {
+      switch(hw_item) {
         case 2000: i = pr_default; break;
         case 2001: i = pr_all; break;
         case 2003: i = pr_cpu; break;
@@ -619,6 +625,9 @@ void do_hw(hd_data_t *hd_data, FILE *f, hd_hw_item_t hw_item)
       case arch_x86_64:
         s = "X86_64";
         break;
+      case arch_aarch64:
+        s = "AArch64";
+        break;
     }
 
     switch(b_arch) {
@@ -651,6 +660,9 @@ void do_hw(hd_data_t *hd_data, FILE *f, hd_hw_item_t hw_item)
       case boot_grub:
         t = "grub";
         break;
+      case boot_uboot:
+        t = "uboot";
+        break;
     }
 
     fprintf(f ? f : stdout, "Arch: %s/%s\n", s, t);
@@ -785,22 +797,53 @@ void do_test(hd_data_t *hd_data)
 void help()
 {
   fprintf(stderr,
-    "Usage: hwinfo [options]\n"
+    "Usage: hwinfo [OPTIONS]\n"
     "Probe for hardware.\n"
-    "  --short        just a short listing\n"
-    "  --log logfile  write info to logfile\n"
-    "  --debug level  set debuglevel\n"
-    "  --version      show libhd version\n"
-    "  --dump-db n    dump hardware data base, 0: external, 1: internal\n"
-    "  --hw_item      probe for hw_item\n"
-    "  hw_item is one of:\n"
-    "    all, bios, block, bluetooth, braille, bridge, camera, cdrom, chipcard, cpu,\n"
-    "    disk, dsl, dvb, floppy, framebuffer, gfxcard, hub, ide, isapnp, isdn,\n"
-    "    joystick, keyboard, memory, modem, monitor, mouse, netcard, network,\n"
-    "    partition, pci, pcmcia, pcmcia-ctrl, pppoe, printer, scanner, scsi, smp,\n"
-    "    sound, storage-ctrl, sys, tape, tv, usb, usb-ctrl, vbe, wlan, zip\n\n"
-    "  Note: debug info is shown only in the log file. (If you specify a\n"
-    "  log file the debug level is implicitly set to a reasonable value.)\n"
+    "Options:\n"
+    "    --<HARDWARE_ITEM>\n"
+    "        This option can be given more than once. Probe for a particular\n"
+    "        HARDWARE_ITEM. Available hardware items are:\n"
+    "        all, arch, bios, block, bluetooth, braille, bridge, camera,\n"
+    "        cdrom, chipcard, cpu, disk, dsl, dvb, fingerprint, floppy,\n"
+    "        framebuffer, gfxcard, hub, ide, isapnp, isdn, joystick, keyboard,\n"
+    "        memory, modem, monitor, mouse, netcard, network, partition,\n"
+    "        pci, pcmcia, pcmcia-ctrl, pppoe, printer, redasd,\n"
+    "        reallyall, scanner, scsi, smp, sound, storage-ctrl, sys, tape,\n"
+    "        tv, uml, usb, usb-ctrl, vbe, wlan, xen, zip\n"
+    "    --short\n"
+    "        Show only a summary. Use this option in addition to a hardware\n"
+    "        probing option.\n"
+    "    --listmd\n"
+    "        Normally hwinfo does not report RAID devices. Add this option to\n"
+    "        see them.\n"
+    "    --only DEVNAME\n"
+    "        This option can be given more than once. If you add this option,\n"
+    "        only data about devices with DEVNAME will be shown.\n"
+    "    --save-config SPEC\n"
+    "        Store config  for a particular device below /var/lib/hardware.\n"
+    "        SPEC can be a device name, an UDI, or 'all'. This option must be\n"
+    "        given in addition to a hardware probing option.\n"
+    "    --show-config UDI\n"
+    "        Show saved config data for a particular device.\n"
+    "    --map\n"
+    "        If disk names have  changed (e.g. after a kernel update) this\n"
+    "        prints a list of disk name mappings. Note  that  you must have\n"
+    "        used --save-config at some point before for this can work.\n"
+    "    --debug N\n"
+    "        Set debug level to N. The debug info is shown only in the log\n"
+    "        file. If you specify a log file, the debug level is implicitly\n"
+    "        set to a reasonable value.\n"
+    "    --verbose\n"
+    "        Increase verbosity. Only together with --map.\n"
+    "    --log FILE\n"
+    "        Write log info to FILE.\n"
+    "    --dump-db N\n"
+    "        Dump hardware data base. N is either 0 for the external data\n"
+    "        base in /var/lib/hardware, or 1 for the internal data base.\n"
+    "    --version\n"
+    "        Print libhd version.\n"
+    "    --help\n"
+    "        Print usage.\n"
   );
 }
 
@@ -1084,7 +1127,6 @@ int x11_install_info(hd_data_t *hd_data)
   driver_info_t *di;
   char *x11i;
   int fb_mode, kbd_ok = 0;
-  unsigned yast2_color = 0;
   char *xkbrules = NULL, *xkbmodel = NULL, *xkblayout = NULL;
   char *xserver, *version, *busid;
   driver_info_t *x11_driver;
@@ -1115,24 +1157,6 @@ int x11_install_info(hd_data_t *hd_data)
 
   xserver = get_xserver(hd_data, &version, &busid, &x11_driver);
 
-  switch(hd_mac_color(hd_data)) {
-    case 0x01:
-      yast2_color = 0x5a4add;
-      break;
-    case 0x04:
-      yast2_color = 0x32cd32;
-      break;
-    case 0x05:
-      yast2_color = 0xff7f50;
-      break;
-    case 0x07:
-      yast2_color = 0x000000;
-      break;
-    case 0xff:
-      yast2_color = 0x7f7f7f;
-      break;
-  }
-
   if(hd_data->progress) {
     printf("\r%64s\r", "");
     fflush(stdout);
@@ -1666,8 +1690,8 @@ unsigned map_fill(map_t *map, hd_data_t *hd_data, hd_t *hd_manual)
   for(i = 0; i < map_len; i++) {
     if(map[i].model) {
       map[i].model_ok = 1;
-      for(j = i + 1; j < map_len; j++) {
-        if(map[j].model && !strcmp(map[i].model, map[j].model)) {
+      for(j = 0; j < map_len; j++) {
+        if(j != i && map[j].model && !strcmp(map[i].model, map[j].model)) {
           map[i].model_ok = 0;
           break;
         }
@@ -1676,8 +1700,8 @@ unsigned map_fill(map_t *map, hd_data_t *hd_data, hd_t *hd_manual)
 
     if(map[i].serial) {
       map[i].serial_ok = 1;
-      for(j = i + 1; j < map_len; j++) {
-        if(map[j].serial && !strcmp(map[i].serial, map[j].serial)) {
+      for(j = 0; j < map_len; j++) {
+        if(j != i && map[j].serial && !strcmp(map[i].serial, map[j].serial)) {
           map[i].serial_ok = 0;
           break;
         }
@@ -1686,8 +1710,8 @@ unsigned map_fill(map_t *map, hd_data_t *hd_data, hd_t *hd_manual)
 
     if(map[i].size) {
       map[i].size_ok = 1;
-      for(j = i + 1; j < map_len; j++) {
-        if(map[i].size == map[j].size) {
+      for(j = 0; j < map_len; j++) {
+        if(j != i && map[i].size == map[j].size) {
           map[i].size_ok = 0;
           break;
         }
