@@ -1048,8 +1048,8 @@ typedef struct s_pci_t {
   char *sysfs_bus_id;				/**< sysfs bus id */
   char *modalias;				/**< module alias */
   char *label;					/**< Consistant Device Name (CDN), pci firmware spec 3.1, chapter 4.6.7 */
-  unsigned edid_len[4];				/**< edid record length */
-  unsigned char edid_data[4][0x80];		/**< edid record */
+  unsigned edid_len[6];				/**< edid record length */
+  unsigned char edid_data[6][0x80];		/**< edid record */
 } pci_t;
 
 /**
@@ -1181,6 +1181,8 @@ typedef struct devtree_s {
   char *path, *filename;
   unsigned pci:1;
   char *name, *model, *device_type, *compatible;
+  char *ccin, *fru_number, *loc_code, *serial_number, *part_number;
+  char *description;
   int class_code;                       /**< class : sub_class : prog-if */
   int vendor_id, device_id, subvendor_id, subdevice_id;
   int revision_id, interrupt;
@@ -1408,6 +1410,7 @@ typedef struct {
  */
 typedef struct {
   unsigned manu_year;
+  unsigned manu_week;
   unsigned min_vsync, max_vsync;	/**< vsync range */
   unsigned min_hsync, max_hsync;	/**< hsync range */
   unsigned clock;			/**< pixel clock in kHz */
