@@ -25,6 +25,7 @@
 #define PROC_ISAPNP		"/proc/isapnp"
 #define PROC_PARTITIONS		"/proc/partitions"
 #define PROC_APM		"/proc/apm"
+#define PROC_XEN_BALLOON	"/proc/xen/balloon"
 
 #define DEV_NVRAM		"/dev/nvram"
 #define DEV_PSAUX		"/dev/psaux"
@@ -42,6 +43,7 @@
 #define PROG_RMMOD		"/sbin/rmmod"
 #define PROG_CARDCTL		"/sbin/cardctl"
 #define PROG_UDEVINFO		"/usr/bin/udevinfo"
+#define PROG_UDEVADM		"/sbin/udevadm"
 
 #define KLOG_BOOT		"/var/log/boot.msg"
 #define ISAPNP_CONF		"/etc/isapnp.conf"
@@ -57,33 +59,7 @@
 #endif
 
 #define PROGRESS(a, b, c) progress(hd_data, a, b, c)
-#define ADD2LOG(a...) str_printf(&hd_data->log, -2, a)
-
-#undef LIBHD_MEMCHECK
-
-#if defined(__i386__) || defined(__PPC__)
-/*
- * f: function we are in
- * a: first argument
- */
-
-#ifdef __i386__
-#define CALLED_FROM(f, a) ((void *) ((unsigned *) &a)[-1] - 5)
-#endif
-
-#ifdef __PPC__
-/* (1-arg funcs only) #define CALLED_FROM(f, a) ((void *) *((unsigned *) ((void *) &a - ((short *) f)[1] - 4)) - 4) */
-static inline void *getr1() { void *p; asm("mr %0,1" : "=r" (p) :); return p; }
-#define CALLED_FROM(f, a) ((void *) ((unsigned *) (getr1() - ((short *) f)[1]))[1] - 4)
-#endif
-#else
-#undef LIBHD_MEMCHECK
-#endif
-
-#ifdef LIBHD_MEMCHECK
-FILE *libhd_log;
-#endif
-
+#define ADD2LOG(a...) hd_log_printf(hd_data, a)
 
 /*
  * define to make (hd_t).unique_id a hex string, otherwise it is a
@@ -128,6 +104,10 @@ char *canon_str(char *, int);
 
 int hex(char *string, int digits);
 
+void hd_log(hd_data_t *hd_data, char *buf, ssize_t len);
+void hd_log_printf(hd_data_t *hd_data, char *format, ...) __attribute__ ((format (printf, 2, 3)));            
+void hd_log_hex(hd_data_t *hd_data, int with_ascii, unsigned data_len, unsigned char *data);
+
 void str_printf(char **buf, int offset, char *format, ...) __attribute__ ((format (printf, 3, 4)));
 void hexdump(char **buf, int with_ascii, unsigned data_len, unsigned char *data);
 str_list_t *search_str_list(str_list_t *sl, char *str);
@@ -221,12 +201,13 @@ int hd_report_this(hd_data_t *hd_data, hd_t *hd);
 str_list_t *hd_module_list(hd_data_t *hd_data, unsigned id);
 
 char* get_sysfs_attr(const char* bus, const char* device, const char* attr);
-char* get_sysfs_attr_by_path(const char* path, const char* attr);
+char *get_sysfs_attr_by_path(const char *path, const char *attr);
+char *get_sysfs_attr_by_path2(const char *path, const char *attr, unsigned *len);
 
-hal_device_t *hd_free_hal_devices(hal_device_t *dev);
 void hd_pci_complete_data(hd_t *hd);
 void hd_pci_read_data(hd_data_t *hd_data);
 
+hal_device_t *hd_free_hal_devices(hal_device_t *dev);
 char *hd_hal_print_prop(hal_prop_t *prop);
 
 void hal_invalidate(hal_prop_t *prop);
@@ -239,12 +220,19 @@ hal_prop_t *hal_get_list(hal_prop_t *prop, const char *key);
 char *hal_get_useful_str(hal_prop_t *prop, const char *key);       
 
 hal_device_t *hal_find_device(hd_data_t *hd_data, char *udi);
+hal_prop_t *hal_add_new(hal_prop_t **prop);
 
 char *hd_get_hddb_dir(void);
 char *hd_get_hddb_path(char *sub);
 
 int hd_mod_cmp(char *str1, char *str2);
 
+int get_probe_val_int(hd_data_t *hd_data, enum probe_feature feature);
+char *get_probe_val_str(hd_data_t *hd_data, enum probe_feature feature);
+str_list_t *get_probe_val_list(hd_data_t *hd_data, enum probe_feature feature);
+
+str_list_t *sort_str_list(str_list_t *sl0, int (*cmp_func)(const void *, const void *));
+
 
 #ifdef __cplusplus
 }
