@@ -7,7 +7,7 @@
 #include <ctype.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/pci.h>
+#include <linux/pci.h>
 
 #include "hd.h"
 #include "hd_int.h"
@@ -60,6 +60,7 @@ static void add_xen_storage(hd_data_t *hd_data);
 static void hd_read_virtio(hd_data_t *hd_data);
 static void hd_read_uisvirtpci(hd_data_t *hd_data);
 static void hd_read_ibmebus(hd_data_t *hd_data);
+static void add_edid_from_file(const char *file, pci_t *pci, int index, hd_data_t *hd_data);
 
 void hd_scan_sysfs_pci(hd_data_t *hd_data)
 {
@@ -128,9 +129,10 @@ void hd_pci_read_data(hd_data_t *hd_data)
   str_list_t *sl;
   char *s;
   pci_t *pci;
-  int fd, i;
-  str_list_t *sf_bus, *sf_bus_e;
-  char *sf_dev;
+  int fd;
+  str_list_t *sf_bus, *sf_bus_e, *sf_drm_dirs, *sf_drm_dir, *sf_drm_subdirs,
+    *sf_drm_subdir;
+  char *sf_dev, *sf_drm = NULL, *sf_drm_subpath = NULL, *sf_drm_edid = NULL;
 
   sf_bus = read_dir("/sys/bus/pci/devices", 'l');
 
@@ -269,33 +271,44 @@ void hd_pci_read_data(hd_data_t *hd_data)
       close(fd);
     }
 
+    /* FIXME: stil valid? */
     for(u = 0; u < sizeof pci->edid_len / sizeof *pci->edid_len; u++) {
       str_printf(&s, 0, "%s/edid%u", sf_dev, u + 1);
-      if((fd = open(s, O_RDONLY)) != -1) {
-        pci->edid_len[u] = read(fd, pci->edid_data[u], sizeof pci->edid_data[u]);
+      add_edid_from_file(s, pci, u, hd_data);
+    }
+    s = free_mem(s);
 
-        ADD2LOG("    edid%u[%u]\n", u + 1, pci->edid_len[u]);
+    /* try searching the monitor data in <PCI_dev>/drm/x/x/edid files if no data found*/
+    if (pci->edid_len[0] == 0) {
+      str_printf(&sf_drm, 0, "%s/drm", sf_dev);
+      u = 0;
 
-        if(pci->edid_len[u] > 0) {
-          for(i = 0; i < sizeof pci->edid_data[u]; i += 0x10) {
-            ADD2LOG("      ");
-            hd_log_hex(hd_data, 1, 0x10, pci->edid_data[u] + i);
-            ADD2LOG("\n");
+      /* get <PCI_dev>/drm/x listing */
+      sf_drm_dirs = read_dir(sf_drm, 'd');
+      for(sf_drm_dir = sf_drm_dirs; sf_drm_dir; sf_drm_dir = sf_drm_dir->next) {
+        str_printf(&sf_drm_subpath, 0, "%s/drm/%s", sf_dev, sf_drm_dir->str);
+
+        /* get <PCI_dev>/drm/x/x listing */
+        sf_drm_subdirs = read_dir(sf_drm_subpath, 'd');
+        for(sf_drm_subdir = sf_drm_subdirs; sf_drm_subdir; sf_drm_subdir = sf_drm_subdir->next) {
+          /* try loading <PCI_dev>/drm/x/x/edid file */
+          str_printf(&sf_drm_edid, 0, "%s/%s/edid", sf_drm_subpath, sf_drm_subdir->str);
+          add_edid_from_file(sf_drm_edid, pci, u, hd_data);
+
+          if (pci->edid_len[u] > 0) {
+            u = u + 1;
           }
         }
-        else {
-          pci->edid_len[u] = 0;
-        }
 
-        close(fd);
-      }
-      else {
-        pci->edid_len[u] = 0;
+        free_str_list(sf_drm_subdirs);
       }
+
+      sf_drm_subpath = free_mem(sf_drm_subpath);
+      sf_drm_edid = free_mem(sf_drm_edid);
+      sf_drm = free_mem(sf_drm);
+      free_str_list(sf_drm_dirs);
     }
 
-    s = free_mem(s);
-
     pci->rev = pci->data[PCI_REVISION_ID];
 
     if((pci->addr_flags[6] & IORESOURCE_MEM)) {
@@ -312,6 +325,31 @@ void hd_pci_read_data(hd_data_t *hd_data)
   free_str_list(sf_bus);
 }
 
+void add_edid_from_file(const char *file, pci_t *pci, int index, hd_data_t *hd_data) {
+  int fd, i;
+
+  if((fd = open(file, O_RDONLY)) != -1) {
+    if (index < sizeof pci->edid_len / sizeof *pci->edid_len) {
+      pci->edid_len[index] = read(fd, pci->edid_data[index], sizeof pci->edid_data[index]);
+      ADD2LOG("    found edid file at %s (size: %d)\n", file, pci->edid_len[index]);
+
+      if(pci->edid_len[index] > 0) {
+        for(i = 0; i < sizeof pci->edid_data[index]; i += 0x10) {
+          ADD2LOG("      ");
+          hd_log_hex(hd_data, 1, 0x10, pci->edid_data[index] + i);
+          ADD2LOG("\n");
+        }
+      }
+    }
+    else {
+      ADD2LOG("    monitor list full, ignoring monitor data %s\n", file);
+    }
+    close(fd);
+  }
+  else {
+    pci->edid_len[index] = 0;
+  }
+}
 
 void add_pci_data(hd_data_t *hd_data)
 {
@@ -878,10 +916,12 @@ void add_mv643xx_eth(hd_data_t *hd_data, char *entry, char *platform_type)
  */
 void hd_read_platform(hd_data_t *hd_data)
 {
-  char *s, *platform_type;
+  char *s, *platform_type, *device_type = NULL;
   str_list_t *sf_bus, *sf_bus_e, *sf_eth_dev = NULL;
   char *sf_dev, *sf_eth_net;
   int mv643xx_eth_seen = 0;
+  int is_net, is_storage, is_usb;
+  hd_t *hd;
 
   sf_bus = read_dir("/sys/bus/platform/devices", 'l');
 
@@ -901,21 +941,70 @@ void hd_read_platform(hd_data_t *hd_data)
 
     if((s = get_sysfs_attr_by_path(sf_dev, "modalias"))) {
       platform_type = canon_str(s, strlen(s));
-      ADD2LOG("    type = \"%s\"\n", platform_type);
+      device_type = free_mem(device_type);
+      if((s = get_sysfs_attr_by_path(sf_dev, "uevent"))) {
+        char *t, *t2;
+        if((t = strstr(s, "OF_NAME="))) {
+          t += sizeof "OF_NAME=" - 1;
+          if((t2 = strchr(t, '\n'))) *t2 = 0;
+          device_type = strdup(t);
+        }
+      }
+      ADD2LOG("    type = \"%s\", modalias = \"%s\"\n", device_type, platform_type);
+      is_net = 0;
       sf_eth_net = new_str(hd_read_sysfs_link(sf_dev, "net"));
       sf_eth_dev = read_dir(sf_eth_net, 'd');
-      ADD2LOG("  platform device: sf_eth_net = %s sf_eth_dev = %p\n", sf_eth_net, sf_eth_dev);
-      if (
+      is_net = sf_eth_net && sf_eth_dev;
+      is_storage = device_type && !strcmp(device_type, "sata");
+      is_usb = device_type && !strcmp(device_type, "usb");
+      if(is_net) ADD2LOG("    is net: sf_eth_net = %s\n", sf_eth_net);
+      if(is_storage) ADD2LOG("    is storage\n");
+      if(is_usb) ADD2LOG("    is usb\n");
+      free_mem(sf_eth_net);
+      free_str_list(sf_eth_dev);
+      if(
         /* there is 'mv643xx_eth.0', 'mv643xx_eth.1' and 'mv643xx_eth_shared.' */
-        sf_eth_dev && sf_eth_net &&
+        is_net &&
         strstr(platform_type, "mv643xx_eth") &&
         !mv643xx_eth_seen++
       ) {
         add_mv643xx_eth(hd_data, sf_bus_e->str, platform_type);
       }
+      else if(is_net) {
+        hd = add_hd_entry(hd_data, __LINE__, 0);
+        hd->base_class.id = bc_network;
+        hd->sub_class.id = 0;
+        str_printf(&hd->device.name, 0, "ARM Ethernet %d", hd->slot);
+        hd->modalias = new_str(platform_type);
+        hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+        hd->sysfs_bus_id = new_str(sf_bus_e->str);
+        s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+        if(s) add_str_list(&hd->drivers, s);
+      }
+      else if(is_storage) {
+        hd = add_hd_entry(hd_data, __LINE__, 0);
+        hd->base_class.id = bc_storage;
+        hd->sub_class.id = sc_sto_ide;
+        str_printf(&hd->device.name, 0, "ARM SATA %d", hd->slot);
+        hd->modalias = new_str(platform_type);
+        hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+        hd->sysfs_bus_id = new_str(sf_bus_e->str);
+        s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+        if(s) add_str_list(&hd->drivers, s);
+      }
+      else if(is_usb) {
+        hd = add_hd_entry(hd_data, __LINE__, 0);
+        hd->base_class.id = bc_serial;
+        hd->sub_class.id = sc_ser_usb;
+
+        str_printf(&hd->device.name, 0, "ARM USB %d", hd->slot);
+        hd->modalias = new_str(platform_type);
+        hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+        hd->sysfs_bus_id = new_str(sf_bus_e->str);
+        s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+        if(s) add_str_list(&hd->drivers, s);
+      }
       free_mem(platform_type);
-      free_mem(sf_eth_net);
-      free_str_list(sf_eth_dev);
     }
 
     free_mem(sf_dev);
@@ -1563,6 +1652,7 @@ void hd_read_virtio(hd_data_t *hd_data)
   free_str_list(sf_bus);
 }
 
+
 /*
  * uisvirtpci
  */
@@ -1634,5 +1724,6 @@ void hd_read_uisvirtpci(hd_data_t *hd_data)
 
   free_str_list(sf_bus);
 }
+
 /** @} */
 
