@@ -5,8 +5,13 @@
 #include "hd_int.h"
 #include "smbios.h"
 
-
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+/**
+ * @defgroup SMBIOSint System Management BIOS (SMBIOS)
+ * @ingroup libhdINFOint
+ * @brief System Management BIOS functions
+ *
+ * @{
+ */
 
 enum sm_map_type { sm_map_str, sm_map_num2str };
 
@@ -851,7 +856,7 @@ void smbios_parse(hd_data_t *hd_data)
           sm->connect.i_type.id = sm_data[5];
           sm->connect.x_type.id = sm_data[7];
           sm->connect.port_type.id = sm_data[8];
-          smbios_id2str(&sm->connect.port_type, &smbios_connect_conn_type, 0xff);
+          smbios_id2str(&sm->connect.i_type, &smbios_connect_conn_type, 0xff);
           smbios_id2str(&sm->connect.x_type, &smbios_connect_conn_type, 0xff);
           smbios_id2str(&sm->connect.port_type, &smbios_connect_port_type, 0xff);
         }
@@ -1660,3 +1665,5 @@ void smbios_dump(hd_data_t *hd_data, FILE *f)
   }
 }
 
+/** @} */
+
