@@ -374,6 +374,10 @@ void set_class_entries(hd_data_t *hd_data, hd_t *hd, usb_t *usb)
   }
 
   switch(cls) {
+    case 1:
+      hd->base_class.id = bc_multimedia;
+      hd->sub_class.id = sc_multi_audio;
+      break;
     case 2:
       if(usb->i_sub == 6 && usb->i_prot == 0) {
         hd->base_class.id = bc_network;
@@ -447,6 +451,10 @@ void set_class_entries(hd_data_t *hd_data, hd_t *hd, usb_t *usb)
       hd->base_class.id = bc_hub;
       break;
 
+    case 0x0b:
+      hd->base_class.id = bc_chipcard;
+      break;
+
     case 0xe0:
       if(sub == 1 && prot == 1) {
         hd->base_class.id = bc_bluetooth;
@@ -508,7 +516,7 @@ void add_input_dev(hd_data_t *hd_data, char *name)
     hd_sysfs_id(name)
   );
 
-  if(!strncmp(sf_cdev_name, "ts", sizeof "ts" - 1)) return;
+  if(!sf_cdev_name || !strncmp(sf_cdev_name, "ts", sizeof "ts" - 1)) return;
 
   if((s = get_sysfs_attr_by_path(name, "dev"))) {
     if(sscanf(s, "%u:%u", &u1, &u2) == 2) {
@@ -755,7 +763,6 @@ void read_usb_lp(hd_data_t *hd_data, hd_t *hd)
   char *s;
   char buf[1024];
   int fd, two_ints[2];
-  unsigned bus;
   str_list_t *sl0, *sl;
   char *vend, *prod, *serial, *descr;
 
@@ -767,7 +774,6 @@ void read_usb_lp(hd_data_t *hd_data, hd_t *hd)
   }
   
   ADD2LOG("  usb/lp: bus = %d, dev_nr = %d\n", two_ints[0], two_ints[1]);
-  bus = ((two_ints[0] & 0xff) << 8) + (two_ints[1] & 0xff);
 
   if(ioctl(fd, LPIOC_GET_VID_PID(sizeof two_ints), two_ints) != -1) {
     /* just for the record */
