@@ -10,6 +10,22 @@
 
 #include "../hd/hddb_int.h"
 
+typedef enum hw_item {
+  hw_none = 0, hw_sys, hw_cpu, hw_keyboard, hw_braille, hw_mouse,
+  hw_joystick, hw_printer, hw_scanner, hw_chipcard, hw_monitor, hw_tv,
+  hw_display, hw_framebuffer, hw_camera, hw_sound, hw_storage_ctrl,
+  hw_network_ctrl, hw_isdn, hw_modem, hw_network, hw_disk, hw_partition,
+  hw_cdrom, hw_floppy, hw_manual, hw_usb_ctrl, hw_usb, hw_bios, hw_pci,
+  hw_isapnp, hw_bridge, hw_hub, hw_scsi, hw_ide, hw_memory, hw_dvb,
+  hw_pcmcia, hw_pcmcia_ctrl, hw_ieee1394, hw_ieee1394_ctrl, hw_hotplug,
+  hw_hotplug_ctrl, hw_zip, hw_pppoe, hw_wlan, hw_redasd, hw_dsl, hw_block,
+  hw_tape, hw_vbe, hw_bluetooth,
+  /* append new entries here */
+  hw_unknown, hw_all                                    /* hw_all must be last */
+} hd_hw_item_t;
+
+#include "../hd/hwclass_names.h"
+
 #define TAG_PCI		1	/* pci ids */
 #define TAG_EISA	2	/* eisa ids */
 #define TAG_USB		3	/* usb ids */
@@ -200,6 +216,7 @@ struct option options[] = {
   { "no-compact", 0, NULL, 13},
   { "join-keys-first", 0, NULL, 14},
   { "combine", 0, NULL, 15},
+  { "no-range", 0, NULL, 16},
   { }
 };
 
@@ -220,6 +237,7 @@ struct {
   unsigned no_compact:1;
   unsigned join_keys_first:1;
   unsigned combine:1;		/* always combine driver info */
+  unsigned no_range:1;		/* don't create entries with ranges */
   char *logfile;
   char *outfile;
   char *cfile;
@@ -305,6 +323,10 @@ int main(int argc, char **argv)
         opt.combine = 1;
         break;
 
+      case 16:
+        opt.no_range = 1;
+        break;
+
       default:
         fprintf(stderr,
           "Usage: check_hd [options] files\n"
@@ -822,10 +844,10 @@ hddb_entry_mask_t add_entry(skey_t *skey, hddb_entry_t idx, char *val)
 {
   hddb_entry_mask_t e_mask = 0;
   int i;
-  unsigned id, tag, range, mask;
+  unsigned id, tag, range, mask, u, u1;
   char *s, *s1, *s2, c;
   hid_t *hid;
-  str_t *str;
+  str_t *str, *str0;
 
   for(i = 0; (unsigned) i < sizeof hddb_is_numeric / sizeof *hddb_is_numeric; i++) {
     if(idx == hddb_is_numeric[i]) break;
@@ -837,7 +859,25 @@ hddb_entry_mask_t add_entry(skey_t *skey, hddb_entry_t idx, char *val)
     /* numeric id */
     e_mask |= 1 << idx;
 
-    i = parse_id(val, &id, &tag, &range, &mask);
+    /* special */
+    if(idx == he_hwclass) {
+      tag = id = 0;
+
+      str0 = split('|', val);
+      for(u1 = 0, str = str0; str && u1 <= 16; str = str->next) {
+        u = hd_hw_item_type(str->str);
+        if(u) {
+          id += u << u1;
+          u1 += 8;
+        }
+      }
+      free_str(str0, 1);
+
+      i = 1;
+    }
+    else {
+      i = parse_id(val, &id, &tag, &range, &mask);
+    }
 
     // printf("parse_id = %d\n", i);
 
@@ -1077,7 +1117,7 @@ void write_id(FILE *f, hddb_entry_t ent, hid_t *hid)
   static char *tag_name[6] = { "", "pci ", "eisa ", "usb ", "special ", "pcmcia " };
   int tag;
   unsigned u;
-  char c;
+  char c, *s;
 
   switch(hid->any.flag) {
     case FLAG_ID:
@@ -1086,7 +1126,15 @@ void write_id(FILE *f, hddb_entry_t ent, hid_t *hid)
         fprintf(stderr, "internal oops\n");
         exit(2);
       }
-      if(tag == TAG_EISA && (ent == he_vendor_id || ent == he_subvendor_id)) {
+      if(ent == he_hwclass) {
+        /* is special */
+        for(u = (hid->num.id & 0xffffff); u; u >>= 8) {
+          s = hd_hw_item_name(u & 0xff);
+          if(s) fprintf(f, "%s", s);
+          if(u > 0x100) fprintf(f, "|");
+        }
+      }
+      else if(tag == TAG_EISA && (ent == he_vendor_id || ent == he_subvendor_id)) {
         fprintf(f, "%s", eisa_str(hid->num.id));
       }
       else {
@@ -1785,6 +1833,8 @@ int combine_keys(skey_t *skey0, skey_t *skey1)
   /* no mask value */
   if(hid0->num.has.mask || hid1->num.has.mask) return 0;
 
+  if(opt.no_range) return 0;
+
   /* must be adjacent ranges, can overlap  */
   r0 = hid0->num.has.range ? hid0->num.range : 1;
   r1 = hid1->num.has.range ? hid1->num.range : 1;
@@ -2818,7 +2868,7 @@ void write_cfile(FILE *f, list_t *hd)
     hddb.strings_len, hddb.strings_len
   );
 
-  fprintf(f, "hddb_list_t hddb_internal_list[%u] = {\n", hddb.list_len);
+  fprintf(f, "static hddb_list_t hddb_internal_list[%u] = {\n", hddb.list_len);
   for(u = 0; u < hddb.list_len; u++) {
     fprintf(f,
     "  { 0x%08x, 0x%08x, 0x%08x, 0x%08x }%s\n",
@@ -2829,7 +2879,7 @@ void write_cfile(FILE *f, list_t *hd)
   }
   fprintf(f, "};\n\n");
 
-  fprintf(f, "unsigned hddb_internal_ids[%u] = {\n", hddb.ids_len);
+  fprintf(f, "static unsigned hddb_internal_ids[%u] = {\n", hddb.ids_len);
   for(u = 0; u < hddb.ids_len; u++) {
     if((u % 6) == 0) fputc(' ', f);
     fprintf(f, " 0x%08x", hddb.ids[u]);
@@ -2840,7 +2890,7 @@ void write_cfile(FILE *f, list_t *hd)
 
   qstr = quote_string(hddb.strings, hddb.strings_len);
   qstr_len = qstr ? strlen(qstr) : 0;
-  fprintf(f, "char hddb_internal_strings[%u] = \"\\\n", hddb.strings_len);
+  fprintf(f, "static char hddb_internal_strings[%u] = \"\\\n", hddb.strings_len);
   for(u = 0; u < qstr_len; ) {
     len = qstr_len - u;
     if(len > 72) len = 72;
